default_app_config = 'django_ledger.apps.DjangoLedgerConfig'


"""Django Ledger"""
__version__ = '0.2.9'
__license__ = 'GPLv3 License'

__author__ = 'Miguel Sanda'
__email__ = 'msanda@arrobalytics.com'

__url__ = 'https://arrobalytics.com'


from django.contrib import admin

from django_ledger.models import (LedgerModel, EntityModel, ChartOfAccountModel, AccountModel,
                                  JournalEntryModel, TransactionModel, EntityManagementModel, InvoiceModel,
                                  BillModel)


class TransactionModelInLine(admin.TabularInline):
    model = TransactionModel


class JournalEntryModelAdmin(admin.ModelAdmin):
    readonly_fields = [
        'ledger'
    ]
    inlines = [
        TransactionModelInLine
    ]

    class Meta:
        model = JournalEntryModel


class EntityManagementInLine(admin.TabularInline):
    model = EntityManagementModel


class EntityModelAdmin(admin.ModelAdmin):
    inlines = [
        EntityManagementInLine
    ]

    class Meta:
        model = EntityModel


class AccountModelAdmin(admin.ModelAdmin):
    actions = [
        'activate',
        'inactivate',
        'lock',
        'unlock'
    ]
    actions_on_top = True
    actions_on_bottom = True
    sortable_by = [
        'code'
    ]
    list_display = [
        '__str__',
        'active',
        'locked'
    ]
    list_filter = [
        'role',
        'balance_type',
    ]

    class Meta:
        model = AccountModel

    def activate(self, request, queryset):
        queryset.update(active=True)

    def inactivate(self, request, queryset):
        queryset.update(active=False)

    def lock(self, request, queryset):
        queryset.update(locked=True)

    def unlock(self, request, queryset):
        queryset.update(locked=False)

    # def role_bs(self, acc):
    #     return acc.role_bs.upper()

    # role_bs.short_description = 'Balance Sheet Role'


class AccountsModelInLine(admin.TabularInline):
    model = AccountModel
    readonly_fields = [
        'role_bs',
        'role',
        'balance_type',
    ]


class ChartOfAccountsModelAdmin(admin.ModelAdmin):
    inlines = [
        AccountsModelInLine
    ]

    class Meta:
        model = ChartOfAccountModel


class LedgerModelAdmin(admin.ModelAdmin):
    class Meta:
        model = LedgerModel


class InvoiceModelAdmin(admin.ModelAdmin):
    readonly_fields = [
        'invoice_number',
        'due_date',
        'ledger'
    ]

    class Meta:
        model = InvoiceModel


class BillModelAdmin(admin.ModelAdmin):
    readonly_fields = [
        'bill_number',
        'due_date',
        'ledger'
    ]

    class Meta:
        model = BillModel


admin.site.register(EntityModel, EntityModelAdmin)
admin.site.register(JournalEntryModel, JournalEntryModelAdmin)
admin.site.register(LedgerModel, LedgerModelAdmin)
admin.site.register(ChartOfAccountModel, ChartOfAccountsModelAdmin)
admin.site.register(AccountModel, AccountModelAdmin)
admin.site.register(InvoiceModel, InvoiceModelAdmin)
admin.site.register(BillModel, BillModelAdmin)



from django.urls import path

from . import views

app_name = 'django_ledger'

urlpatterns = [

    # Entity Views ----
    path('entity/',
         views.EntityModelListView.as_view(),
         name='entity-list'),
    path('entity/create/',
         views.EntityModelCreateView.as_view(),
         name='entity-create'),
    path('entity/set-default/',
         views.SetDefaultEntityView.as_view(),
         name='entity-set-default'),
    path('entity/<slug:entity_slug>/',
         views.EntityModelDetailVew.as_view(),
         name='entity-detail'),
    path('entity/<slug:entity_slug>/set-date/',
         views.SetDateView.as_view(),
         name='entity-set-date'),
    path('entity/<slug:entity_slug>/update/',
         views.EntityModelUpdateView.as_view(),
         name='entity-update'),

    # Financial Statements ---
    path('entity/<slug:entity_slug>/balance-sheet/',
         views.EntityModelBalanceSheetView.as_view(),
         name='entity-bs'),
    path('entity/<slug:entity_slug>/income-statement/',
         views.EntityModelIncomeStatementView.as_view(),
         name='entity-ic'),

    # Entity Chart of Accounts -----
    path('coa/<slug:entity_slug>/<slug:coa_slug>/update/',
         views.ChartOfAccountsUpdateView.as_view(),
         name='coa-update'),

    # Accounts ---
    path('account/<slug:entity_slug>/<slug:coa_slug>/',
         views.AccountModelListView.as_view(),
         name='account-list'),
    path('account/<slug:entity_slug>/<slug:coa_slug>/create/',
         views.AccountModelCreateView.as_view(),
         name='account-create'),
    path('account/<slug:entity_slug>/<slug:coa_slug>/<int:account_pk>/update/',
         views.AccountModelUpdateView.as_view(),
         name='account-update'),

    # Ledger Views ----
    path('ledger/<slug:entity_slug>/',
         views.LedgerModelListView.as_view(),
         name='ledger-list'),
    path('ledger/<slug:entity_slug>/create/',
         views.LedgerModelCreateView.as_view(),
         name='ledger-create'),
    path('ledger/<slug:entity_slug>/<slug:ledger_pk>/update/',
         views.LedgerModelUpdateView.as_view(),
         name='ledger-update'),
    path('ledger/<slug:entity_slug>/<slug:ledger_pk>/balance-sheet/',
         views.LedgerBalanceSheetView.as_view(),
         name='ledger-bs'),
    path('ledger/<slug:entity_slug>/<slug:ledger_pk>/income-statement/',
         views.LedgerIncomeStatementView.as_view(),
         name='ledger-ic'),

    # Journal Entry Views ----
    path('journal-entry/<slug:entity_slug>/<slug:ledger_pk>/',
         views.JournalEntryListView.as_view(),
         name='je-list'),
    path('journal-entry/<slug:entity_slug>/<slug:ledger_pk>/create/',
         views.JournalEntryCreateView.as_view(),
         name='je-create'),
    path('journal-entry/<slug:entity_slug>/<slug:ledger_pk>/<int:je_pk>/',
         views.JournalEntryDetailView.as_view(),
         name='je-detail'),
    path('journal-entry/<slug:entity_slug>/<slug:ledger_pk>/<int:je_pk>/update/',
         views.JournalEntryUpdateView.as_view(),
         name='je-update'),

    # TXS ----
    path('txs/<slug:entity_slug>/<slug:ledger_pk>/journal-entry/<int:je_pk>/',
         views.TXSJournalEntryView.as_view(),
         name='txs'),
    path('txs/<slug:entity_slug>/<slug:coa_slug>/account/<int:account_pk>/',
         views.TXSAccountView.as_view(),
         name='txs-account'),

    # INVOICES ----
    path('invoice/<slug:entity_slug>/',
         views.InvoiceModelListView.as_view(),
         name='invoice-list'),
    path('invoice/<slug:entity_slug>/create/',
         views.InvoiceModelCreateView.as_view(),
         name='invoice-create'),
    path('invoice/<slug:entity_slug>/<slug:invoice_slug>/update/',
         views.InvoiceModelUpdateView.as_view(),
         name='invoice-update'),

    # Bills ----
    path('bill/<slug:entity_slug>/',
         views.BillModelListView.as_view(),
         name='bill-list'),
    path('bill/<slug:entity_slug>/create/',
         views.BillModelCreateView.as_view(),
         name='bill-create'),
    path('bill/<slug:entity_slug>/<slug:bill_slug>/update/',
         views.BillModelUpdateView.as_view(),
         name='bill-update'),

    path('dashboard/', views.DashboardView.as_view(), name='dashboard'),

    # Auth Views ---
    path('accounts/login/', views.DjangoLedgerLoginView.as_view(), name='login'),
    path('accounts/logout/', views.DjangoLedgerLogoutView.as_view(), name='logout'),

    path('', views.RootUrlView.as_view(), name='root-url')
]



from setuptools import setup

import django_ledger

setup(
    name='django-ledger',
    version=django_ledger.__version__,
    packages=['django_ledger'],
    url=django_ledger.__url__,
    license=django_ledger.__license__,
    keywords='django, finance, accounting, balance sheet, income statement, general ledger, money, engine',
    author=django_ledger.__author__,
    author_email=django_ledger.__email__,
    description='Financial analysis backend for Django. Balance Sheet, Income Statements, Chart of Accounts, Entities',
    install_requires=[
        'django',
        'django-mptt'
    ],
    project_urls={
        # 'Bug Tracker': 'https://bugs.example.com/HelloWorld/',
        # 'Documentation': 'https://docs.example.com/HelloWorld/',
        'Source Code': 'https://github.com/arrobalytics/django-ledger',
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Office/Business :: Financial :: Accounting',
        'Development Status :: 3 - Alpha',
        'Framework :: Django :: 3.0',
        'Intended Audience :: Financial and Insurance Industry',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
    ]
)
